/* -*- C -*-
 * Spamtest connection library
 * File: msgstore.h
 *      (C)  Alex Tutubalin <lexa@lexa.ru>
 *	Apr-May 2003
 *
 * Created: Wed Apr 23 22:08:08 2003
 */
#ifndef _MSGSTORE_H
#define _MSGSTORE_H
#include <sys/types.h>

#ifdef __cplusplus
extern "C"
{
#endif


struct __msg_heap_t;

typedef struct 
{
    char	*hptr;
    size_t      length;
}some_text;



#define HDR_CREATED	1
#define HDR_NAMEDONE	2
#define HDR_VALUESTART	3
#define HDR_VALUE	4
#define HDR_VALUEDONE	5

#define HDR_USE_CRLF    1

typedef struct 
{
    some_text	header;
    some_text	value;
    int		state;
    int		deleted;
}msg_header;

typedef  struct 
{
    msg_header *array;
    size_t	used;
    size_t	alloc;
    struct __msg_heap_t  *storeptr;
    char       *hdrbuf;
    size_t      bufalloc;
    size_t      bufused;

    unsigned int flags;
}headers_list_t;

typedef struct {
    some_text *array;
    size_t         used;
    size_t         alloc;
    int		self_mem;
    struct __msg_heap_t  *storeptr;
} recipient_list_t;

typedef struct {
    char	*chunk;
    size_t	alloc;
    size_t	used;
} body_chunk_t;

typedef struct {
    body_chunk_t *array;
    size_t	alloc;
    size_t	used;
    long	totalsize;
    struct __msg_heap_t  *store;
    size_t	maxrss;
    int		file_fd;
    char	*tempdir;
    char        *mmap_ptr;
    size_t      mmap_len;
} message_text_t;

typedef struct  
{
    some_text from;
    some_text ip_addr;

    recipient_list_t *recipients;

    headers_list_t *headers;
	
    struct __msg_heap_t *store;
    message_text_t *bodystore;
    int headers_processed;
    int last_string_complete;

} message_t;

typedef void (* writer_func_t)(const char*, size_t len);
typedef void (* writer_func_ex_t)(void *context, const char*, size_t len);

    
message_text_t *msgtext_init(struct __msg_heap_t *heap);
message_text_t *msgtext_init_with_tempfile(struct __msg_heap_t *heap, 
					   const char *dir,size_t max_rss);
long msgtext_totalsize(message_text_t *msgt);
int msgtext_count_chunks(message_text_t *msgt);
int msgtext_chunk_len(message_text_t *msgt, size_t chunk);
char* msgtext_get_chunk(message_text_t *msgt, size_t chunk);
void msgtext_callback(message_text_t *msgt, writer_func_t func);
void msgtext_callback_ex(message_text_t *msgt, writer_func_ex_t func, void *context);
int msgtext_puttext(message_text_t *msgt, const char *data, size_t len);
void msgtext_close(message_text_t *msgt);




/* recipient list methods */
recipient_list_t *rcptlist_init(struct __msg_heap_t *heap);
void rcptlist_clean(recipient_list_t*);
int rcptlist_add(recipient_list_t*,const char*);
int rcptlist_exists(recipient_list_t*,const char*);
int rcptlist_addn(recipient_list_t *rl, const char *rcpt,size_t len);
int rcptlist_adduniq(recipient_list_t *rl, const char *rcpt);
int rcptlist_count(recipient_list_t*);
char* rcptlist_getn(recipient_list_t*,int i);
int  rcptlist_iterate(recipient_list_t*, writer_func_t);
int  rcptlist_iterate_ex(recipient_list_t*, writer_func_ex_t, void *context);



/* create new message structure */
message_t *message_init(void);
message_t *message_init_with_heap(struct __msg_heap_t *heap);
message_t *message_init_with_tempfile(const char*tempdir, size_t maxrss);

/* delete all allocated data */
void message_free(message_t *msg);

/* set message envelope from */
int message_set_from(message_t *msg, const char *from);

/* set message envelope IP (relay) address */
int message_set_ip(message_t *msg, const char *ip);

/* clean message envelope recinient list */
void message_clean_rcpt(message_t *msg);

/* add envelope recipient to list */
int message_add_rcpt(message_t *msg, const char *rcpt);


/* store some bytes of data (entire message or less) into message
 * internal store */
int message_put_body(message_t *msg, char *data,size_t len);


/* read message parameters: */

/* access to body */
long get_message_bodysize(message_t *msgt);
int get_message_bodychunks_count(message_t *msgt);
int get_message_bodychunk_len(message_t *msgt,size_t chunk);
char* get_message_bodychunk_ptr(message_t *msgt,size_t chunk);
void message_body_callback(message_t *msgt, writer_func_t func);
void message_body_callback_ex(message_t *msgt, writer_func_ex_t func, void *context);


/* get recipient */
char *get_message_from(message_t *msg);

/* get envelope IP (as string) */
char *get_message_ip(message_t *msg);

/* get recipient count */
int get_message_nrcpt(message_t*);
/* get i-th recipient */
char *get_message_rcpt(message_t *msg, int i);
/* iterate on recipients */
int message_rcpts_iterate(message_t *msg, writer_func_t callback);
int message_rcpts_iterate_ex(message_t *msg, writer_func_ex_t callback, void *context);

/* access to headers: */

/* this callback type will be called with two parameters: header name
   and header value (with all continuation characters included */
typedef void (* header_iterator_t)(char* hname, char *hval);
typedef void (* header_iterator_ex_t)(void *context, char* hname, char *hval);

/* iterate on all message headers, return number of headers */
int message_headers_iterate(message_t *msg, header_iterator_t callback);
int message_headers_iterate_ex(message_t *msg, header_iterator_ex_t callback, void *context);


/* iterate on separate header list, return number of headers */
int headers_iterate(headers_list_t *msg, header_iterator_t callback);
int headers_iterate_ex(headers_list_t *msg, header_iterator_ex_t callback, void *context);


/* Header operations */
headers_list_t* headers_init(struct __msg_heap_t *heap);
headers_list_t * headers_clone(message_t *msg);
void headers_free(headers_list_t *hdr);
int headers_del_hdr(headers_list_t *,const char *header);
int headers_add_hdr(headers_list_t *,const char *header, const char *value);
int headers_new_hdr(headers_list_t *,const char *header,const char *value);
int headers_chg_hdr(headers_list_t *hdr,const char *header,const char *value);
int headers_prepend_hdr(headers_list_t *hdr, const char *header, const char *value);

/* E-mail address manipulations */
int rfc822_unquote_one(char *dest,unsigned int size, char *from, char *defdomain);
int rfc822_quote_one(char *dest,unsigned int size, char *from, char *defdomain);
recipient_list_t *rfc822_unquote_list(recipient_list_t *from,char *defdomain);
recipient_list_t *rfc822_quote_list(recipient_list_t *from,char *defdomain);

#ifdef __cplusplus
}
#endif

#endif
